/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;
using System.Collections;
using System.Collections.Generic;


namespace Sagittarius.Metadata
{
	/// <summary>
	/// A section of MetadataElements
	/// Containing a sectionnname as well
	/// </summary>
	public class MetadataSection
	{
		protected List<MetadataElement> elementsList;
		private string sectionName;

#region konstruktorok

		public MetadataSection (string sectionName)
		{
			//Console.WriteLine("Entering MetadataSection constructor(string sectionName). SectionName: "+sectionName);
			this.sectionName = sectionName;
			this.elementsList = new List<MetadataElement> ();
			//Console.WriteLine("Leaving MetadataSection constructor(string sectionName)");
		}
		
		public MetadataSection ()
		{
			//Console.WriteLine("Entering MetadataSection constructor()");
			this.elementsList = new List<MetadataElement> ();
			//Console.WriteLine("Leaving MetadataSection constructor()");
		}

#endregion

#region property metódusok

		public string SectionName 
		{
			get { return sectionName; }
			set { sectionName = value; }
		}
		
		public int Length
		{
			get { return elementsList.Count; }
		}

#endregion

#region element access methods

		public virtual void AddElement (MetadataElement newElement)
		{
			//Console.WriteLine("Entering MetadataSection.AddElement. Element: "+newElement.Key);
			bool alreadyPresent = false;
			foreach(MetadataElement element in elementsList)
			{
				if (element.CompareTo(newElement)==0)
				{
					alreadyPresent=true;
					break;
				}
			}
			if (!alreadyPresent)
			{
				elementsList.Add(newElement);
			}
			else
			{
				throw new Exception ("MetadataElement "+newElement.Key+" is already present in "+sectionName+" section. Cannot be added once more.");
			}
			//Console.WriteLine("Leaving MetadataSection.AddElement");
		}

		public List<MetadataElement> Elements
		{
			get { return elementsList; }
		}

		/*public virtual MetadataElement GetElement (string key)
		{
			//Console.WriteLine("Entering MetadataSection.GetElement");
			MetadataElement lookUp = new MetadataElement(key,"");
			MetadataElement result = null;
			foreach (MetadataElement element in elementsList)
			{
				if (element.CompareTo(lookUp)==0)
				{
					result = element;
				}
				if (result==null)
				{
					Console.WriteLine("##############");
				}
			}
			//Console.WriteLine("Leaving MetadataSection.GetElement");
			return result;
		}*/
		
		public virtual string GetValue(string key)
		{
			MetadataElement elementToLookup = new MetadataElement(key,"");
			string result="";
			foreach(MetadataElement element in elementsList)
			{
				if(element.CompareTo(elementToLookup)==0)
				{
					result = element.Value;
				}
			}
			return result;
		}
		
		public virtual void SetElement (string key, string newValue)
		{
			foreach (MetadataElement element in elementsList)
			{
				if (element.Key==key)
				{
					element.Value=newValue;
				}
			}
		}
		
#endregion

		public override string ToString ()
		{
			string output;
			output = "Metadata section:\nSection name: "+sectionName + "\n";
			foreach ( MetadataElement element in elementsList)
			{
				output += element.ToString()+"\n";
			}
			return output;
		}
		
		public virtual MetadataSection Copy()
		{
			MetadataSection result = new MetadataSection();
			foreach (MetadataElement element in elementsList)
			{
				result.AddElement(element.Copy());
			}
			return result;
		}
	}
}
