/* Copyright 2008 Daniel Vokó <vokoda@gmail.com>
 * 
 * This file is part of Sagittarius.
 *
 * Sagittarius is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * Sagittarius is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Sagittarius.  If not, see <http://www.gnu.org/licenses/>.
 */


using System;

namespace Sagittarius
{
	public class Config
	{
		private GConf.Client gconfClient;	// GConf client for reading configuration information
		// GConfig keys
		private static string KEY_BASE = "/apps/sagittarius";
		private static string KEY_THUMBSIZE = KEY_BASE + "/thumb_size";
		private static string KEY_EXIFTOOL_EXECUTABLE = KEY_BASE + "/exiftool_executable";
		private static string KEY_CREATE_BACKUP = KEY_BASE + "/create_backup";
		//private static string KEY_THUMB_QUAL = KEY_BASE +"/thumb_qual";
		private static string KEY_PREV_MAX_SIZE = KEY_BASE +"/prev_max_size";
		//private static string KEY_PREV_QUAL = KEY_BASE +"/prev_qual";
		// Gconfig values
		private int thumbSize;
		private string exiftoolExecutable;
		private bool createBackup=true;
		//private int thumbQual;
		private int prevMaxSize;
		//private int prevQual;

		public Config()
		{
			//Console.WriteLine("Entering Config.Config.");
			// Initialization of the GConf client, 
			// listener attaching to the changes of config keys, 
			// and the  reading of the configuration
			gconfClient = new GConf.Client();
			gconfClient.AddNotify (KEY_BASE, new GConf.NotifyEventHandler (GConfChanged));
			ReadConfig();
			//Console.WriteLine("Leaving Config.Config.");
		}
		/// <summary>
		/// Callback called when the sagittarius records of the GConf database has changed
		/// Tinkering in preferences dialog ends in calling this callback;
		/// </summary>
		private void GConfChanged(object sender,GConf.NotifyEventArgs args)
		{
			//Console.WriteLine("Entering Config.GConfChanged.");
			ReadConfig();
			//Console.WriteLine("Leaving Config.GConfChanged.");
		}
		/// <summary>
		/// This method reads all user.configurable settings from the GConf database
		/// </summary>
		public void ReadConfig()
		{
			//Console.WriteLine("Entering Config.ReadConfig.");
			try
			{
				thumbSize = (int)gconfClient.Get(KEY_THUMBSIZE);
				exiftoolExecutable= (string)gconfClient.Get(KEY_EXIFTOOL_EXECUTABLE);
				createBackup = (bool)gconfClient.Get(KEY_CREATE_BACKUP);
				//thumbQual=(int)gconfClient.Get(KEY_THUMB_QUAL);
				prevMaxSize=(int)gconfClient.Get(KEY_PREV_MAX_SIZE);
				//prevQual = (int)gconfClient.Get(KEY_PREV_QUAL);
			}
			catch (GConf.NoSuchKeyException e )
			{
				Console.WriteLine("Something bad happened while reading the GConf database! It seems that there are no configuration settings!");
				Console.WriteLine ("Exception message: " + e.Message);
				Console.WriteLine("Writing default configuration settings...");
				gconfClient.Set(KEY_THUMBSIZE,120);
				gconfClient.Set(KEY_EXIFTOOL_EXECUTABLE,"/usr/bin/exiftool");
				gconfClient.Set(KEY_CREATE_BACKUP,false);
				//gconfClient.Set(KEY_THUMB_QUAL,1);
				gconfClient.Set(KEY_PREV_MAX_SIZE,400);
				//gconfClient.Set(KEY_PREV_QUAL,1);
			}
			catch // here shall be a InvalidCaseException handle as well, because GConf values are all objects, and can be altered by anybody
			{
				Console.WriteLine("Something major happened while reading the GConf database!");
			}
			//Console.WriteLine("Leaving Config.ReadConfig.");
		}
		
		/// <summary>
		/// Write configuration information to GConf database
		/// </summary>
		public void WriteConfig()
		{
			//Console.WriteLine("Entering Config.WriteConfig.");
			try
			{
				gconfClient.Set (KEY_THUMBSIZE,thumbSize);
				gconfClient.Set(KEY_EXIFTOOL_EXECUTABLE,exiftoolExecutable);
				gconfClient.Set(KEY_CREATE_BACKUP,createBackup);
				//gconfClient.Set(KEY_THUMB_QUAL,thumbQual);
				gconfClient.Set(KEY_PREV_MAX_SIZE,prevMaxSize);
				//gconfClient.Set(KEY_PREV_QUAL,prevQual);
			}
			catch // here shall be a InvalidCaseException handle as well, because GConf values are all objects, and can be altered by anybody
			{
				Console.WriteLine("Something major happened while writing the GConf database!");
			}
			//Console.WriteLine("Leaving Config.WriteConfig.");
		}
		
#region property

		public int ThumbSize
		{
			get { return thumbSize; }
			set { thumbSize = value; }
		}
		public string ExitoolExecutable
		{
			get { return exiftoolExecutable; }
			set { exiftoolExecutable = value; }
		}
		public bool CreateBackup
		{
			get { return createBackup; }
			set { createBackup = value; }
		}
		/*public int ThumbQual
		{
			get { return thumbQual; }
			set { thumbQual = value; }
		}*/
		public int PrevMaxSize
		{
			get { return prevMaxSize; }
			set { prevMaxSize = value; }
		}
		/*public int PrevQual
		{
			get { return prevQual; }
			set { prevQual = value; }
		}*/

#endregion
	}
}
